﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Windows.Forms;

namespace HeuristicLab.Optimization.Networks.Views {
  public partial class EntitySelector : UserControl {
    private IEntity root;
    public IEntity Root {
      get { return root; }
      set {
        root = value;
        PopulateTreeView();
      }
    }
    private IEntity selectedEntity;
    public IEntity SelectedEntity {
      get {
        return selectedEntity;
      }
      private set {
        if (selectedEntity != value) {
          selectedEntity = value;
          OnSelectedEntityChanged();
        }
      }
    }
    public TreeView EntitiesTreeView {
      get { return treeView; }
    }

    public EntitySelector() {
      InitializeComponent();
    }
    public EntitySelector(IEntity root) : this() {
      Root = root;
    }

    public event EventHandler SelectedEntityChanged;
    protected virtual void OnSelectedEntityChanged() {
      var handler = SelectedEntityChanged;
      if (handler != null) SelectedEntityChanged(this, EventArgs.Empty);
    }


    private void PopulateTreeView() {
      treeView.Nodes.Clear();
      treeView.Nodes.Add(CreateTreeNode(Root));
    }
    private TreeNode CreateTreeNode(IEntity entity) {
      if (entity == null)
        return new TreeNode("null");
      else {
        TreeNode node = new TreeNode(entity.Name);
        node.Tag = entity;
        foreach (var child in entity.Children)
          node.Nodes.Add(CreateTreeNode(child));
        return node;
      }
    }

    private void treeView_AfterSelect(object sender, TreeViewEventArgs e) {
      if (treeView.SelectedNode != null)
        SelectedEntity = (IEntity)treeView.SelectedNode.Tag;
      else
        SelectedEntity = null;
    }
  }
}
